// pp.RES is created using Microsoft toolchain rc
//
// rc pp.rc

#define PP_MANIFEST_FILEFLAGS 0
#include <windows.h>

CREATEPROCESS_MANIFEST_RESOURCE_ID RT_MANIFEST "winres\\pp.manifest"

VS_VERSION_INFO VERSIONINFO
    FILEVERSION        0,0,0,0
    PRODUCTVERSION     0,0,0,0
    FILEFLAGSMASK      VS_FFI_FILEFLAGSMASK
    FILEFLAGS          PP_MANIFEST_FILEFLAGS
    FILEOS             VOS_NT_WINDOWS32
    FILETYPE           VFT_APP
    FILESUBTYPE        VFT2_UNKNOWN
BEGIN
    BLOCK "StringFileInfo"
    BEGIN
        BLOCK "000004B0"
        BEGIN
            VALUE "CompanyName", " \0"
            VALUE "FileDescription", " \0"
            VALUE "FileVersion", "0.0.0.0\0"
            VALUE "InternalName", " \0"
            VALUE "LegalCopyright", " \0"
            VALUE "LegalTrademarks", " \0"
            VALUE "OriginalFilename", " \0"
            VALUE "ProductName", " \0"
            VALUE "ProductVersion", "0.0.0.0\0"
        END
    END
    BLOCK "VarFileInfo"
    BEGIN
        VALUE "Translation", 0x00, 0x04B0
    END
END

WINEXE ICON winres\\pp.ico

